<script src="https://cdnjs.cloudflare.com/ajax/libs/require.js/2.3.7/require.min.js"></script>

<script>
// Passing the search options to the search.js file
const SEARCH_OPTIONS = JSON.parse('{{ theme_static_search | tojson | safe }}');
const SEARCH_FILE = "{{ pathto('_static/search.json', 1) }}";
const ADVANCE_SEARCH_PATH = "{{ pathto('search.html', 1) }}";
const EXTRA_SOURCES = JSON.parse('{{ theme_search_extra_sources | tojson | safe }}');
</script>

{% if pagename != "search" %}
<script src="{{ pathto('_static/js/search.js', 1) }}"></script>
{% endif %}
