{# Sphinx sidebar template: "show source" link. #}
{%- if show_source and has_source and sourcename %}
  <div role="note" aria-label="source link">
    <ul class="this-page-menu">
      <li><a href="{{ pathto('_sources/' + sourcename, true)|e }}"
            rel="nofollow">{{ _('Show Source') }}</a></li>
    </ul>
   </div>
{%- endif %}
